<?php
/*
	Plugin Name: Image Wall
	Plugin URI: http://www.themodernnomad.com/image-wall-plugin/#utm_campaign=Image_Wall&utm_source=wordpress&utm_medium=website&utm_content=plugin_link
	Description: Browse posts/pages by their images, displayed randomly on an infinitely scrollable page. The images link back to where they are attached.
	Version: 0.2
	Author: Gustav Andersson
	Author URI: http://www.themodernnomad.com/#utm_campaign=Image_Wall&utm_source=wordpress&utm_medium=website&utm_content=author_link
*/
/*  Copyright 2012  Gustav Andersson  (email : mail@themodernnomad.com)

    There are three parts of the license of the Image Wall. The first part is 
    the bit that I have written, which is everything in image-wall.css, image-wall.js 
    image-wall.php. Those items are copyrighted under the GPLv2 license described below.
    
    The 'isotope' code under the 'isotope' folder is licensed as described at 
    http://isotope.metafizzy.co. Note that you need to buy a license to use isotope
    on a commercial website, which means you need to get that license before you can
    use the Image Wall on a commercial site too, as 'isotope' is used by 'image wall'.
    
    The 'Infinite Scroll' code (anything under the 'infinite' directory) is 
    licensed and protected as described at http://www.infinite-scroll.com/
    
    GPLv2:

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as 
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/************************************************************************/
/*			PLUGIN ACTIVATION				*/
/************************************************************************/
register_activation_hook(__FILE__,'tmn_iw_plugin_activation');
function tmn_iw_plugin_activation()
{
//	do_action( 'iw_attachment_hash_regenerate' );
//	wp_schedule_event(current_time( 'timestamp' ), 'daily', 'iw_attachment_hash_regenerate');
	wp_schedule_event(time(), 'daily', 'iw_attachment_hash_regenerate');
} 


/************************************************************************/
/*			PLUGIN DE-ACTIVATION				*/
/************************************************************************/

register_deactivation_hook(__FILE__,'tmn_iw_plugin_deactivation');
function tmn_iw_plugin_deactivation()
{
	wp_clear_scheduled_hook('iw_attachment_hash_regenerate');
	//wp_schedule_single_event(current_time( 'timestamp'), 'iw_attachment_hash_clear' );
	do_action( 'iw_attachment_hash_clear' );
}


/************************************************************************/
/*			ATTACHMENT HASHING				*/
/************************************************************************/

add_action('iw_attachment_hash_regenerate','tmn_iw_attachment_hash_regenerate');
add_action('iw_attachment_hash_clear','tmn_iw_attachment_hash_clear');
add_action('add_attachment','tmn_iw_attachment_hash_regenerate');


/* Regenerates the Image Wall hash, salted by the time of the request to simulate randomness. If the argument is a valid attachment_id, only that attachment is updated. If it is empty, all attachments are regenerated. Else, an error is thrown. 
   Returns true on success, false otherwise, or mayeb a cheerful error.
*/
function tmn_iw_attachment_hash_regenerate($attachment_id) {
	if( !isset($attachment_id) ) {
		$myposts = get_posts( array(
			'post_type' 	=> 'attachment',
			'nopaging' 	=> true,
			'post_status' 	=> 'all',
			'post_parent' 	=> null
		) );
		$tmn_time = time();
		foreach( $myposts as $mypost ) {
			update_post_meta($mypost->ID, 'tmn-iw-hash', md5($mypost->ID + $tmn_time));
		}
		return true;
	} else {
		return update_post_meta($attachment_id, 'tmn-iw-hash', md5($attachment_id));
	}
}

/* Clears all the Image Wall time hashes from all attachments. */
function tmn_iw_attachment_hash_clear() {
	$myposts = get_posts( array(
		'post_type' 	=> 'attachment',
		'nopaging' 	=> true,
		'post_status' 	=> 'all',
		'post_parent' 	=> null
	) );
	foreach( $myposts as $mypost ) {
		delete_post_meta($mypost->ID, 'tmn-iw-hash' );
	}
}


/************************************************************************/
/*			ENQUEUE ON SHORTCODE USAGE			*/
/************************************************************************/

function tmn_iw_enqueue() {
	global $post;
	if (   preg_match('/(?<!\[)\[image_wall.*?\]/', $post->post_content) ) {
		wp_enqueue_script('tmniwjs', plugins_url( 'image-wall.js'  , __FILE__ ), array('jquery'), '1', true);
		wp_enqueue_style ('tmniwcss', plugins_url( 'image-wall.css' , __FILE__ ), array(), '1');
		add_filter('body_class','tmn_iw_body_class');
	}
}    

function tmn_iw_body_class($classes) {
	$classes[] = 'image-wall';	
	return $classes;
}

add_action('wp_enqueue_scripts', 'tmn_iw_enqueue');


/************************************************************************/
/*			SHORTCODE	 				*/
/************************************************************************/

/* Example usage: 

	[image_wall column_width="300" image_sizes="medium,large" column_proportion_restrictions="1.5, 2.0, 3.5" batch_size="30" buffer_pixels="800"  support_author="true" move_to_end="false"]

  Arguments:
	column_width = integer representing pixels, or empty for the smallest image size + 8 (4 pixels border on each side)
	image_sizes = comma-separated list of defined image sizes to include in the image wall. Default value is 'medium'. 
	column_proportion_restrictions = comma-separated list of proportion restrictions. 
		First value is for column 2, second value for column 3 etc. 
		An image must have a x:y proportion equal or higher than the given number to be allowed to span that number of columns. 
		The numbers must be sequentially static or increasing.
		If left blank, no proportion restrictions exist.
		If restrictions for column n exist, but none for n+1, then no image may span more than n columns regardless of proportion.
		Note that the plugin doesn't support images spanning more than 10 columns.
	batch_size = (integer) number of images to bring back per batch. Default: 50
	buffer_pixels = Number of pixels from the bottom before loading of a new batch of images. Default: 2000
	move_to_end = By default, the image wall is displayed where the shortcode is inserted. However, if you set this attribute to 'true', then the Image Wall is moved to the end of the HTML <body> tag. This may fix problems with image-related CSS interference from the rest of the theme. Also, the image wall will be as wide as possible. Unfortunately, the footer is shown above the image wall.
	support_author = Show your appreciation of this plugin! Set this to true to let me place an invisible link to the author's blog and plugin page. Doesn't affect you and greatly helps my search engine ranking. Many thanks! (Default: false)
	open_links_in_new_window = By default, links are opened in new windows. Set this to 'false' to open links in the same window. (Note however, that when you click back to the Image Wall, you start from the start again. 
*/
	
add_shortcode('image_wall', 'image_wall_sc');
function image_wall_sc($atts) {
	global $wp_query;	
	
	extract( shortcode_atts( array(	
		'image_sizes' 		=> 'medium' , 
		'column_width' 		=> '', 
		'batch_size' 		=> '50',
		'buffer_pixels' 	=> '2000',
		'support_author' 	=> 'false',
		'move_to_end'		=> 'false', 
		'column_proportion_restrictions' => '',
		'open_links_in_new_window'	 => 'true'
	), $atts ) );

	// Find, process and verify the image sizes //	
	$image_sizes = preg_split("/[\s,]+/", $image_sizes);
	$erroneous_image_sizes = array_diff($image_sizes, get_intermediate_image_sizes());
	if(! empty($erroneous_image_sizes) ) {
		$output = "<div id='tmn-image-wall-error'><h3>Uh oh! I've detected a problem setting up the Image Wall!</h3><p>The following image sizes are not registered in this wordpress instance. Please remove and try again.</p><ul>";
		foreach ($erroneous_image_sizes as $erroneous_image_size) { $output .= "<li>".$erroneous_image_size."</li>"; }
		$output .= "</ul></div>";
		return $output;
	}

	// This really should be available as a wordpress function!!!!
	global $_wp_additional_image_sizes;

	foreach ( $image_sizes as $s ) {
		$sizes[$s] = array( 'width' => '', 'height' => '', 'crop' => FALSE );
		if ( isset( $_wp_additional_image_sizes[$s]['width'] ) )
			$sizes[$s]['width'] = intval( $_wp_additional_image_sizes[$s]['width'] ); // For theme-added sizes
		else
			$sizes[$s]['width'] = get_option( "{$s}_size_w" ); // For default sizes set in options
		if ( isset( $_wp_additional_image_sizes[$s]['height'] ) )
			$sizes[$s]['height'] = intval( $_wp_additional_image_sizes[$s]['height'] ); // For theme-added sizes
		else
			$sizes[$s]['height'] = get_option( "{$s}_size_h" ); // For default sizes set in options
		if ( isset( $_wp_additional_image_sizes[$s]['crop'] ) )
			$sizes[$s]['crop'] = intval( $_wp_additional_image_sizes[$s]['crop'] ); // For theme-added sizes
		else
			$sizes[$s]['crop'] = get_option( "{$s}_crop" ); // For default sizes set in options
	}
	$image_sizes = $sizes;
	$sizes = null;

	// Find, process and verify the column_width //

	// Find the smallest image size width available
	
	$minimum_width = PHP_INT_MAX;
	foreach ( $image_sizes as $size_name => $size_details) {		
		if($size_details['width'] > 0){
			$minimum_width = min($minimum_width, $size_details['width']);	
		}
	}	
	
	if(empty($column_width) ){
		if ($minimum_width == PHP_INT_MAX) {
			return "<div id='tmn-image-wall-error'><h3>Uh oh! I've detected a problem setting up the Image Wall!</h3><p>None of the given image sizes have a defined width. You must manually enter the column_width in the shortcode as I can't guess it.</p></div>";
		} else {
			$column_width = $minimum_width +8;
		}
	}
	
	if($column_width < 18) {
		return "<div id='tmn-image-wall-error'><h3>Uh oh! I've detected a problem setting up the Image Wall!</h3><p>The resulting column width (".$column_width.") is too small. I require at least a column width of 18px wide.</p></div>";
	}
		
	// Find, process and verify the column proportion restrictions //
	$use_column_proportion_restrictions = !empty($column_proportion_restrictions);
	if($use_column_proportion_restrictions){
		$column_proportion_restrictions = preg_split("/[\s,]+/", "0,".$column_proportion_restrictions);
	}


	// Find, process and verify the batch size //
	$batch_size = intval($batch_size);
	if($batch_size <= 0) {
		return "<div id='tmn-image-wall-error'><h3>Uh oh! I've detected a problem setting up the Image Wall!</h3><p>I couldn't parse the given batch size to a positive integer.</p></div>";		
	}

	// Find, process and verify the batch size //
	$buffer_pixels = intval($buffer_pixels);
	if($buffer_pixels <= 0) {
		return "<div id='tmn-image-wall-error'><h3>Uh oh! I've detected a problem setting up the Image Wall!</h3><p>I couldn't parse the given buffer pixels to a positive integer.</p></div>";		
	}
	
	// Find, process and verify the open links in new windowe //
	if($open_links_in_new_window == 'false') {
		$target_string = '';
	} else {
		$target_string = ' target="_blank" '; 
	}

	
	// Grab and remember the page we are on, if paged.
	$tmn_page = $_GET["tmn_iw_page"] ? $_GET["tmn_iw_page"] : '1' ;
		
	// Done unpacking and verifying the arguments. Let's get some real coding done!
	
	// Create our new image query and run the loop
	$iw_query = new WP_Query( array(
		'post_type' 	 => 'attachment',
		'post_mime_type' =>'image',
		'posts_per_page' => $batch_size,
		'nopaging' 	 => false,
		'post_status' 	 => 'all',
		'post_parent' 	 => null,
		'meta_key' 	 => 'tmn-iw-hash',
		'orderby' 	 => 'meta_value',
		'paged' 	 => $tmn_page
	) );	
	
	if ( ! $iw_query->have_posts() ) {
		if( $tmn_page == 1 ) {
			return "<div id='tmn-image-wall-error'><h3>Uh oh! I have no photos to display!</h3><p>If you just activated the Image Wall plugin, then I am most likely still working on processing your images and generating a random order. Get a cup of tea and check back in an hour. If this error is still here, then <a href='http://www.themodernnomad.com/contact/'>contact me</a> and let me know that you have a problem, and I will take a look at it!</p></div>";
		} else {
			exit(404);
		}
	}
	$output  = '';
	if($support_author) {
		$output .= "<p id='tmn-image-wall-support'>I am a proud supporter of this plugin's developer, <a href='http://www.themodernnomad.com' alt='A blog about living a geo-independent sustainable nomadic life of continuous and indefinite travel.' title='A blog about living a geo-independent sustainable nomadic life of continuous and indefinite travel.'>The Modern Nomad</a>. Visit the <a href='http://www.themodernnomad.com/image-wall-plugin/' alt='A wordpress plugin that allows visitors to browse posts or pages by their images, displayed randomly on an infinitely scrollable Image Wall.' title='A wordpress plugin that allows visitors to browse posts or pages by their images, displayed randomly on an infinitely scrollable Image Wall.'>Image Wall Wordpress Plugin</a> site to find out how to get your own Image Wall.</p>";
	}
	$output .= '<div id="tmn-image-wall"><div id="tmn-image-wall-scroll-messages"></div></div>';
	$output .= '<div id="tmn-image-wall-prep">';
		
	// Loop over each of our fetched attachments.
	while ( $iw_query->have_posts() ) : $iw_query->the_post();
		$parent_ID = get_post_field('post_parent', get_the_id() );
		
		// Don't show orphaned or un-published images.
		if ( !$parent_ID || get_post_field('post_status', $parent_ID ) != 'publish' ) continue; 


		// We need to find the best available image size to use. 
		// That is the image size that will span the most columns (legally) while using the fewest pixels.
		// There must also be a pre-generated version of the image size so we don't use the full image and 
		// totally explode the memory usage out of this thing.

		$columns_spanned_array = array();
		$width_array = array();
		$height_array = array();
		$url_array = array();

		foreach ($image_sizes as $image_size_name => $image_size_details) {				

			$wp_image_details = wp_get_attachment_image_src(get_the_id(), $image_size_name);

			$url    = $wp_image_details[0];
			$width  = $wp_image_details[1];
			$height = $wp_image_details[2];

			$actual_image_details = getimagesize($url);
			if($actual_image_details[0] != $image_size_details['width']) {
				// Uh oh! We are dealing with an image size that hasn't been generated yet. To save the user from downloading potentiall huge full size images,
				// we skip this.
				continue;
			}

			$max_number_of_columns = intval(( $width + 8 ) / $column_width);

			if($use_column_proportion_restrictions){
				for($i = count($column_proportion_restrictions) ; $i > 0 ; $i -- ) {
					if( $width / $height >= $column_proportion_restrictions[$i-1] ) {
						$number_of_columns = min($i, $max_number_of_columns);
						break;
					} else {
					
					}
				}
			} else {
				$number_of_columns = $max_number_of_columns;
			}	

			$columns_spanned_array[] = $number_of_columns;
			$width_array[] = $width;
			$height_array[] = $height;
			$url_array[] = $url;
		}

		// Check that we have at least one viable image version good enough for the wall. 
		if(count($url_array) == 0) continue;

		array_multisort($columns_spanned_array, SORT_ASC, $width_array, SORT_DESC, $url_array, $height_array);

		$columns_spanned = array_pop($columns_spanned_array);
		$width = array_pop($width_array);
		$height = array_pop($height_array);
		$url = array_pop($url_array);

		$alt_and_title = trim(strip_tags(get_the_title() . " (". get_the_title($parent_ID) .")" ));
		
		$output .= '<a class="tmn-image-wall-item-link" href="'.get_permalink( $parent_ID ).'" '.$target_string.' rel="nofollow"><img width="'.$width.'" height="'.$height.'" src="'.$url.'" class="tmn-image-wall-item tmn-image-wall-span-cols-'.$columns_spanned.'" alt="'.$alt_and_title.'" title="'.$alt_and_title.'" /></a>';

	endwhile;
	$output .= '</div>';

	// Our work here is done! Restore the original query and return.
	wp_reset_postdata();
	
	if( $iw_query->max_num_pages > $tmn_page ) {
		$output .= '<a id="tmn-image-wall-next" rel="nofollow" href="' . add_query_arg( 'tmn_iw_page', $tmn_page+1 , get_permalink()) .'">Next</a>';
	}
	
	
	$output .= "\n".'<script type="text/javascript">';
	$output .= "\n".'function tmn_iw_get_column_width(){ return "'.$column_width.'";}';
	$output .= "\n".'function tmn_iw_get_buffer_pixels(){ return "'.$buffer_pixels.'";}';
	$output .= "\n".'function tmn_iw_get_scroll_img_url(){ return "'.plugins_url( 'loading.gif' , __FILE__ ).'";}';
	$output .= "\n".'function tmn_iw_move_image_wall_to_end(){ return "'.$move_to_end.'" == "true";}';
	$output .= "\n"."</script>";
	$output .= "\n"."<style>";
	$output .= "\n".'#tmn-image-wall img.tmn-image-wall-item { height: auto }';
	for($i = 1; $i <= 10; $i++) { 	
		$max_image_width_for_columns_spanned = $column_width * $i - 8;

		$output .= "\n".'#tmn-image-wall img.tmn-image-wall-item.tmn-image-wall-span-cols-'.$i.' { width: '.$max_image_width_for_columns_spanned.'px; }';	
		$output .= "\n".'#tmn-image-wall.tmn-image-wall-cols-'.$i.' img.tmn-image-wall-item { max-width: '.$max_image_width_for_columns_spanned.'px; }';	
	}
	$output .= "\n".'</style>';
	
	return $output;
}


?>